package one.shandan;

import java.lang.String;
import java.lang.Boolean;
import java.util.Iterator;
import java.util.List;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Arrays;
import org.crosswire.jsword.passage.VerseRangeFactory;
import org.crosswire.jsword.versification.system.SystemDefault;
import org.crosswire.jsword.versification.BibleBook;
import org.crosswire.jsword.passage.NoSuchVerseException;
import org.crosswire.jsword.versification.Versification;
import org.crosswire.jsword.versification.system.Versifications;
import org.crosswire.jsword.book.Books;
import org.crosswire.jsword.book.Book;
import org.crosswire.jsword.book.BookException;
import org.crosswire.jsword.passage.Key;
import org.crosswire.jsword.passage.NoSuchKeyException;
import org.jdom2.Attribute;
import org.jdom2.Element;
import org.jdom2.Text;
import org.jdom2.Content;
import org.jdom2.Content.CType;
import org.postgresql.pljava.annotation.Function;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import static org.postgresql.pljava.annotation.Function.Effects.IMMUTABLE;
import static org.postgresql.pljava.annotation.Function.OnNullInput.RETURNS_NULL;
import static org.postgresql.pljava.annotation.Function.Trust.SANDBOXED;

public class PlJavaJSword {
  private static final String DEFAULT_BOOK = "MKJV";
  private static final String DEFAULT_VERSIFICATION = "KJV";
  private static final Versification kjv = Versifications.instance().getVersification(DEFAULT_VERSIFICATION);
  private static final Books BOOKS = Books.installed();
  private static final HashSet<Character> PUNCTUATION = new HashSet<Character>(Arrays.asList('.', ',', ';', ':', '?'));
  private static final Logger logger = LoggerFactory.getLogger(PlJavaJSword.class);

  @Function(onNullInput=RETURNS_NULL, effects=IMMUTABLE, trust=SANDBOXED)
  public static Boolean isValidVerse(String reference) {
    try {
      VerseRangeFactory.fromString(kjv, reference);
      return true;
    } catch (NoSuchVerseException ex) {
    }
    return false;
  }
  @Function(onNullInput=RETURNS_NULL, effects=IMMUTABLE, trust=SANDBOXED)
  public static String normalizeVerse(String reference){
    try {
      return VerseRangeFactory.fromString(kjv, reference).getName();
    } catch (NoSuchVerseException ex) {
      return null;
    }
  }
  private static String parseVerse(Iterator<Content> content){
    String verse = "";
    while (content.hasNext()){
      Content c = content.next();
      if (c.getCType() == CType.Element){
        Element e = (Element)c;
        if (e.getName() != "note"){
          verse += " " + e.getValue().trim();
        }
      } else if (c.getCType() == CType.Text){
        String t = ((Text)c).getValue().trim();
        if (t.isEmpty() || PUNCTUATION.contains(t.charAt(0))){
          verse += t;
        } else {
          verse += " " + t;
        }
      }
    }
    return verse.trim();
  }
  private static String parseContent(Iterator<Content> content)
  {
     List<String> ret = new ArrayList<String>();
     while (content.hasNext()){
       Content c = content.next();
       if (c.getCType() == CType.Element){
         Element e = (Element)c;
         if (e.getName() == "verse"){
           String part = parseVerse(e.getContent().listIterator());
           ret.add(part);
         }
       }
     }
     return String.join(" ", ret);
  }
  @Function(onNullInput=RETURNS_NULL, effects=IMMUTABLE, trust=SANDBOXED)
  public static String getText(String translation, String reference) throws BookException {
    Book book = BOOKS.getBook(translation);
    if (book == null){
      return null;
    }
    try {
      Key key = book.getKey(reference);
      return parseContent(book.getOsisIterator(key, false));
    } catch (NoSuchKeyException ex){
      return null;
    }
  }
  @Function(onNullInput=RETURNS_NULL, effects=IMMUTABLE, trust=SANDBOXED)
  public static String getDefaultText(String reference) throws BookException {
    Book book = BOOKS.getBook(DEFAULT_BOOK);
    if (book == null){
      return null;
    }
    try {
      Key key = book.getKey(reference);
      return parseContent(book.getOsisIterator(key, false));
    } catch (NoSuchKeyException ex){
      return null;
    }
  }
  @Function(effects=IMMUTABLE, trust=SANDBOXED)
  public static String getDefaultTranslation(){
    return DEFAULT_BOOK;
  }
}

